function [ grad_f, grad_f_simple ] = gradient( M, f )
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

n = size(M.VERT,1);
m = size(M.TRIV,1);

if length(f) ~= n
   error('gradient: Number of elements of f is unequal the number of vertices of M.'); 
end

grad_f = zeros(m,3);

% check if g and Dx were calculated

if ~isfield(M, 'Dx')
   M.Dx = trimesh_differential(M); 
end
if ~isfield(M, 'g')
   M.g  = trimesh_fff(M.Dx); 
end

% calculate g^-1

g_inv = zeros(size(M.g));

for k = 1:m
   g_inv(:,:,k) = 1/det(squeeze(M.g(:,:,k))) * [ M.g(2,2,k), -M.g(1,2,k); -M.g(2,1,k), M.g(1,1,k) ];
end

% calculate gradient on the reference triangle (this part has to be
% consistent with the construction of the differential!)

grad_ftilde = zeros(m,2);

for k=1:m
   grad_ftilde(k,1) = f(M.TRIV(k,2)) - f(M.TRIV(k,1));
   grad_ftilde(k,2) = f(M.TRIV(k,3)) - f(M.TRIV(k,1));
end

% gradient = Dx*g^-1*grad_ftilde

for k=1:m
   grad_f(k,:) = squeeze(M.Dx(:,:,k))*squeeze(g_inv(:,:,k))*grad_ftilde(k,:)';
end

% for exercise: gradient without inverse of metric tensor

grad_f_simple = zeros(m,3);

for k=1:m
   grad_f_simple(k,:) = squeeze(M.Dx(:,:,k))*grad_ftilde(k,:)';
end

end

